@@ -13,16 +13,22 @@ __docformat__ = 'restructuredtext'
 import logging
 import os
 import os.path as op
+from functools import partial
+from collections import OrderedDict
+
+
 from datalad.interface.base import Interface
 from datalad.interface.base import build_doc
-from datalad.utils import getpwd
-from datalad.utils import assure_unicode
-from datalad.utils import unlink
+from datalad.utils import (
+    assure_unicode,
+    getpwd,
+    unlink,
+)
 from datalad.dochelpers import exc_str
 from datalad.support.external_versions import external_versions
 from datalad.support.exceptions import CommandError
 from datalad.support.gitrepo import InvalidGitRepositoryError
-
+from datalad.version import __version__, __full_version__
 
 lgr = logging.getLogger('datalad.plugin.wtf')
 
@@ -49,7 +55,6 @@ def get_max_path_length(top_path=None, maxl=1000):
     """
     if not top_path:
         top_path = getpwd()
-    import os
     import random
     from datalad import lgr
     from datalad.dochelpers import exc_str
@@ -73,7 +78,6 @@ def get_max_path_length(top_path=None, maxl=1000):
 
 
 def _describe_datalad():
-    from datalad.version import __version__, __full_version__
 
     return {
         'version': assure_unicode(__version__),
@@ -108,12 +112,19 @@ def _describe_annex():
 def _describe_system():
     import platform as pl
     from datalad import get_encoding_info
+    from datalad.utils import get_linux_distribution
+    try:
+        dist = get_linux_distribution()
+    except Exception as exc:
+        lgr.warning("Failed to get distribution information: %s", exc_str(exc))
+        dist = tuple()
+
     return {
         'type': os.name,
         'name': pl.system(),
         'release': pl.release(),
         'version': pl.version(),
-        'distribution': ' '.join([_t2s(pl.dist()),
+        'distribution': ' '.join([_t2s(dist),
                                   _t2s(pl.mac_ver()),
                                   _t2s(pl.win32_ver())]).rstrip(),
         'max_path_length': get_max_path_length(getpwd()),
@@ -126,6 +137,14 @@ def _describe_environment():
     return get_envvars_info()
 
 
+def _describe_python():
+    import platform
+    return {
+        'version': platform.python_version(),
+        'implementation': platform.python_implementation(),
+    }
+
+
 def _describe_configuration(cfg, sensitive):
     if not cfg:
         return _HIDDEN
@@ -211,25 +230,53 @@ def _describe_dataset(ds, sensitive):
     from datalad.interface.results import success_status_map
     from datalad.api import metadata
 
-    infos = {
-        'path': ds.path,
-        'repo': ds.repo.__class__.__name__ if ds.repo else None,
+    try:
+        infos = {
+            'path': ds.path,
+            'repo': ds.repo.__class__.__name__ if ds.repo else None,
+            'id': ds.id,
+        }
+        if not sensitive:
+            infos['metadata'] = _HIDDEN
+        elif ds.id:
+            ds_meta = metadata(
+                dataset=ds, reporton='datasets', return_type='list',
+                result_filter=lambda x: x['action'] == 'metadata' and success_status_map[x['status']] == 'success',
+                result_renderer='disabled', on_failure='ignore')
+            if ds_meta:
+                ds_meta = [dm['metadata'] for dm in ds_meta]
+                if len(ds_meta) == 1:
+                    ds_meta = ds_meta.pop()
+                infos['metadata'] = ds_meta
+            else:
+                infos['metadata'] = None
+        return infos
+    except InvalidGitRepositoryError as e:
+        return {"invalid": exc_str(e)}
+
+
+def _describe_location(res):
+    return {
+        'path': res['path'],
+        'type': res['type'],
     }
-    if not sensitive:
-        infos['metadata'] = _HIDDEN
-    elif ds.id:
-        ds_meta = metadata(
-            dataset=ds, reporton='datasets', return_type='list',
-            result_filter=lambda x: x['action'] == 'metadata' and success_status_map[x['status']] == 'success',
-            result_renderer='disabled', on_failure='ignore')
-        if ds_meta:
-            ds_meta = [dm['metadata'] for dm in ds_meta]
-            if len(ds_meta) == 1:
-                ds_meta = ds_meta.pop()
-            infos['metadata'] = ds_meta
-        else:
-            infos['metadata'] = None
-    return infos
+
+
+# Actuall callables for WTF. If None -- should be bound later since depend on
+# the context
+SECTION_CALLABLES = {
+    'datalad': _describe_datalad,
+    'python': _describe_python,
+    'git-annex': _describe_annex,
+    'system': _describe_system,
+    'environment': _describe_environment,
+    'configuration': None,
+    'location': None,
+    'extensions': _describe_extensions,
+    'metadata_extractors': _describe_metadata_extractors,
+    'dependencies': _describe_dependencies,
+    'dataset': None,
+}
 
 
 @build_doc
@@ -257,22 +304,36 @@ class WTF(Interface):
             constraints=EnsureDataset() | EnsureNone()),
         sensitive=Parameter(
             args=("-s", "--sensitive",),
-            constraints=EnsureChoice('some', 'all') | EnsureNone(),
+            constraints=EnsureChoice(None, 'some', 'all'),
             doc="""if set to 'some' or 'all', it will display sections such as 
             config and metadata which could potentially contain sensitive 
             information (credentials, names, etc.).  If 'some', the fields
             which are known to be sensitive will still be masked out"""),
+        sections=Parameter(
+            args=("-S", "--section"),
+            action='append',
+            dest='sections',
+            metavar="SECTION",
+            constraints=EnsureChoice(*sorted(SECTION_CALLABLES)) | EnsureNone(),
+            doc="""section to include.  If not set, all sections.
+            [CMD: This option can be given multiple times. CMD]"""),
+        decor=Parameter(
+            args=("-D", "--decor"),
+            constraints=EnsureChoice('html_details') | EnsureNone(),
+            doc="""decoration around the rendering to facilitate embedding into
+            issues etc, e.g. use 'html_details' for posting collapsable entry
+            to GitHub issues."""),
         clipboard=Parameter(
             args=("-c", "--clipboard",),
             action="store_true",
             doc="""if set, do not print but copy to clipboard (requires pyperclip
-            module"""),
+            module)"""),
     )
 
     @staticmethod
     @datasetmethod(name='wtf')
     @eval_results
-    def __call__(dataset=None, sensitive=None, clipboard=None):
+    def __call__(dataset=None, sensitive=None, sections=None, decor=None, clipboard=None):
         from datalad.distribution.dataset import require_dataset
         from datalad.support.exceptions import NoDatasetArgumentFound
         from datalad.interface.results import get_status_dict
@@ -297,28 +358,35 @@ class WTF(Interface):
         from datalad.ui import ui
         from datalad.support.external_versions import external_versions
 
-        infos = {}
+        infos = OrderedDict()
         res = get_status_dict(
             action='wtf',
-            path=ds.path if ds else op.abspath(op.curdir),
+            path=ds.path if ds else assure_unicode(op.abspath(op.curdir)),
             type='dataset' if ds else 'directory',
             status='ok',
             logger=lgr,
+            decor=decor,
             infos=infos,
         )
-        infos['datalad'] = _describe_datalad()
-        infos['git-annex'] = _describe_annex()
-        infos['system'] = _describe_system()
-        infos['environment'] = _describe_environment()
-        infos['configuration'] = _describe_configuration(cfg, sensitive)
-        infos['extentions'] = _describe_extensions()
-        infos['metadata_extractors'] = _describe_metadata_extractors()
-        infos['dependencies'] = _describe_dependencies()
+
+        # Define section callables which require variables.
+        # so there is no side-effect on module level original
+        section_callables = SECTION_CALLABLES.copy()
+        section_callables['location'] = partial(_describe_location, res)
+        section_callables['configuration'] = \
+            partial(_describe_configuration, cfg, sensitive)
         if ds:
-            try:
-                infos['dataset'] = _describe_dataset(ds, sensitive)
-            except InvalidGitRepositoryError as e:
-                infos['dataset'] = {"invalid": exc_str(e)}
+            section_callables['dataset'] = \
+                partial(_describe_dataset, ds, sensitive)
+        else:
+            section_callables.pop('dataset')
+        assert all(section_callables.values())  # check if none was missed
+
+        if sections is None:
+            sections = sorted(list(section_callables))
+
+        for s in sections:
+            infos[s] = section_callables[s]()
 
         if clipboard:
             external_versions.check(
@@ -334,15 +402,12 @@ class WTF(Interface):
     @staticmethod
     def custom_result_renderer(res, **kwargs):
         from datalad.ui import ui
-        ui.message(_render_report(res))
+        out = _render_report(res)
+        ui.message(out)
 
 
 def _render_report(res):
-    report = u'# WTF\n'
-    report += u'\n'.join(
-        u'- {}: {}'.format(k, v) for k, v in res.items()
-        if k not in ('action', 'infos', 'status')
-    )
+    report = u'# WTF'
 
     def _unwind(text, val, top):
         if isinstance(val, dict):
@@ -362,4 +427,19 @@ def _render_report(res):
         return text
 
     report = _unwind(report, res.get('infos', {}), '')
+
+    decor = res.get('decor', None)
+
+    if not decor:
+        return report
+
+    if decor == 'html_details':
+        report = """\
+<details><summary>DataLad %s WTF (%s)</summary>
+
+%s
+</details>
+        """ % (__version__, ', '.join(res.get('infos', {})), report)
+    else:
+        raise ValueError("Unknown value of decor=%s" % decor)
     return report
